#include "Movable.h"
#include "Status.h"
#include "Vram.h"
#include "Stage.h"
#include "Main.h"
#include "Sprite.h"
#include "Print.h"
#include "Chars.h"

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        *pVram = c; ++pVram;
        ++p;
    }
    return pVram;
}

void PrintStatus() 
{
    PrintS(Vram + 2, "SCORE");
    PrintS(Vram + 16, "HI-SCORE");
    PrintS(Vram + 33, "STAGE");
    PrintByteNumber2(Vram + 39, CurrentStage + 1);
    PrintS(Vram + 43, "TIME");

    {
        static const byte[] RemainChars = {
        	0xca, 0xff, 0xd0, 0xc0,
        };
        ptr<byte> pVram;
        pVram = Vram + 54;
        if (RemainCount > 1) {
            byte i;
            i = RemainCount - 1;
            do {
                ptr<byte> pChar;
                pChar = RemainChars;
                repeat (4) {
                    *pVram = *pChar;
                    ++pVram;
                    ++pChar;
                }
                --i;
            } while (i > 0);
        }
    }

    PrintScore();
    PrintTime();
}

void PrintScore()
{
    ptr<byte> pVram;
    
    pVram = PrintNumber5(Vram + 8, Score);
    *pVram = '0';

    pVram = PrintNumber5(Vram + 25, HiScore);
    *pVram = '0';
}

void PrintTime()
{
    PrintByteNumber3(Vram + 48, StageTime);
}


void PrintGameOver()
{
    PrintS(Vram + VramRowSize * 8 + 27, "GAME OVER");
}

void PrintTimeUp() 
{
    PrintS(Vram + VramRowSize * 8 + 28, "TIME UP");
}

void Title()
{
    static const byte[] TitleBytes = {
        //	L
        0xea, 0xd5, 0xc0, 0xc0, 
        0xea, 0xd5, 0xc0, 0xc0, 
        0xc2, 0xc3, 0xc3, 0xc1, 
        //	I
        0xc2, 0xeb, 0xd7, 0xc1, 
        0xc0, 0xea, 0xd5, 0xc0, 
        0xc2, 0xc3, 0xc3, 0xc1, 
        //	F
        0xea, 0xd7, 0xc3, 0xc1, 
        0xea, 0xd7, 0xc3, 0xc1, 
        0xc2, 0xc1, 0xc0, 0xc0, 
        //	T
        0xc2, 0xeb, 0xd7, 0xc1, 
        0xc0, 0xea, 0xd5, 0xc0, 
        0xc0, 0xc2, 0xc1, 0xc0, 
   };
    constexpr byte LogoLength = 4;
    constexpr byte LogoWidth = 4 * LogoLength;
    constexpr byte LogoX = (VramWidth - LogoWidth) / 2;

    ClearScreen(); 
    HideAllSprites();
    PrintStatus();
    {
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 5 + LogoX;
        p = TitleBytes;
        repeat (LogoLength) {
            repeat (3) {
                repeat (4) {
                    *pVram = *p;
                    ++pVram;
                    ++p;
                }
                pVram += VramRowSize - 4;
            }
            pVram += 4 - VramRowSize * 3;
        }
    }
    PrintS(Vram + VramRowSize * 13 + 25, "PUSH SPACE KEY");
    PrintS(Vram + VramRowSize * 14 + 25, "OR ENTER KEY");
    PrintS(Vram + VramRowSize * 15 + 52, "INUFUTO 2022");
}